unit uMain;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, ExtCtrls, StdCtrls, ComCtrls, Menus, uPolygon;

type
  TDrawingTool = (dtLine, dtRectangle, dtEllipse, dtRoundRect);
  TMain = class(TForm)
    Panel1: TPanel;
    ConnectLastPointBtn: TButton;
    NewPointsBtn: TButton;
    FillBtn: TButton;
    Panel2: TPanel;
    Image: TImage;
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure ConnectLastPointBtnClick(Sender: TObject);
    procedure NewPointsBtnClick(Sender: TObject);
    procedure FillBtnClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    Polygon: TPolygon;
    FirstPoint: boolean;
    PrevPt: TPoint;
    Points: array[0..20] of TPoint;
    PointCount: integer;
    Vehicle: boolean;
    VehiclePosition: TPoint;
    VehiclePositionDone: boolean;
  public
    BrushStyle: TBrushStyle;
    PenStyle: TPenStyle;
    PenWide: Integer;
    Drawing: Boolean;
    Origin, MovePt: TPoint;
    DrawingTool: TDrawingTool;
    CurrentFile: string;
    procedure DrawShape(TopLeft, BottomRight: TPoint; AMode: TPenMode);
  end;

var
  Main: TMain;

implementation

{$R *.DFM}

procedure TMain.FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
   if not Vehicle and (PointCount < 20) then begin
      Origin := Point(X, Y);
      MovePt := Origin;

      if not FirstPoint then begin
         DrawShape(PrevPt, Point(X, Y), pmCopy);
         Drawing := False;
      end;

      FirstPoint := False;
      PrevPt := Origin;
      Points[PointCount] := Origin;
      inc(PointCount);
      if not ConnectLastPointBtn.Enabled then
         ConnectLastPointBtn.Enabled := True;
   end
   else if not VehiclePositionDone and FillBtn.Enabled then begin
      VehiclePosition := Point(X, Y);
      Image.Canvas.Pen.Width := 5;
      move(Points, Polygon.Points, sizeof(Points));
      Polygon.AngleCount := PointCount;
      Polygon.VehiclePosition := VehiclePosition;
      if Polygon.IsDrivingOutsidePolygon then
         Image.Canvas.Pen.Color := clRed
      else
         Image.Canvas.Pen.Color := clGreen;

      DrawShape(VehiclePosition, VehiclePosition, pmCopy);
      Image.Canvas.Pen.Color := clBlack;
      Image.Canvas.Pen.Width := 1;
   end;
end;

procedure TMain.DrawShape(TopLeft, BottomRight: TPoint; AMode: TPenMode);
begin
   with Image.Canvas do begin
      Pen.Mode := AMode;
      case DrawingTool of
         dtLine: begin
            Image.Canvas.MoveTo(TopLeft.X, TopLeft.Y);
            Image.Canvas.LineTo(BottomRight.X, BottomRight.Y);
         end;
         dtRectangle:
            Image.Canvas.Rectangle(TopLeft.X, TopLeft.Y, BottomRight.X, BottomRight.Y);
         dtEllipse:
            Image.Canvas.Ellipse(Topleft.X, TopLeft.Y, BottomRight.X, BottomRight.Y);
         dtRoundRect:
            Image.Canvas.RoundRect(TopLeft.X, TopLeft.Y, BottomRight.X, BottomRight.Y, (TopLeft.X - BottomRight.X) div 2, (TopLeft.Y - BottomRight.Y) div 2);
      end;
   end;
end;

procedure TMain.FormCreate(Sender: TObject);
var
   Bitmap: TBitmap;
begin
   Polygon := TPolygon.Create;
   Bitmap := TBitmap.Create;
   Bitmap.Width := Image.Width;
   Bitmap.Height := Image.Height;
   Image.Picture.Graphic := Bitmap;
   FirstPoint := True;
   PointCount := 0;
   ConnectLastPointBtn.Enabled := False;
   Vehicle := False;
   VehiclePositionDone := False;
end;

procedure TMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
   Polygon.Free;
end;

procedure TMain.ConnectLastPointBtnClick(Sender: TObject);
begin
   if PointCount > 2 then
      DrawShape(Points[PointCount - 1], Points[0], pmCopy);
   TButton(Sender).enabled := False;
   Vehicle := True;
   FillBtn.Enabled := True;
end;

procedure TMain.NewPointsBtnClick(Sender: TObject);
var
   ARect: TRect;
begin
   PointCount := 0;
   FirstPoint := True;
   Vehicle := False;
   VehiclePositionDone := False;
   with Image.Canvas do begin
      CopyMode := cmWhiteness;
      ARect := Rect(0, 0, Image.Width, Image.Height);
      CopyRect(ARect, Image.Canvas, ARect);
      CopyMode := cmSrcCopy;
   end;
   FillBtn.Enabled := False;
end;

procedure TMain.FillBtnClick(Sender: TObject);
var
   x, y: integer;
   Color: TColor;
begin
   Screen.Cursor := crHourGlass;
   NewPointsBtn.Enabled := False;
   FillBtn.Enabled := False;
   move(Points, Polygon.Points, sizeof(Points));
   for y := 0 to Image.Height do begin
      Polygon.VehiclePosition.y := y;
      Image.Update;

      for x := 0 to Image.Width do begin
         Polygon.VehiclePosition.x := x;
         Polygon.AngleCount := PointCount;

         if Polygon.IsDrivingOutsidePolygon then
            Color := clYellow
         else if not Polygon.CouldNotResolve then
            Color := clBlue
         else
            Color := clRed;

         if Image.Canvas.Pixels[x, y] <> clBlack then
            Image.Canvas.Pixels[x, y] := Color;
      end;
   end;
   FillBtn.Enabled := True;
   NewPointsBtn.Enabled := True;
   Screen.Cursor := crDefault;
end;

end.
