unit uPolygon;

interface

uses
  Windows;

type
  TCoord = record
    X: integer;
    Y: integer;
  end;

type
  TAngles = record
    NoOfPoints: integer;
    Points: array [0..50] of TCoord
  end;

type
  TPolygon = class
  private
    Angles: TAngles;
    SearchPoint: TCoord;
    Polygon: array [0..50] of TCoord;
    procedure ReadCoordinates;
    procedure RemoveAngle(AngleNo: integer);
    function  UpperXAxis(P1, P2: TCoord): boolean;
    function  YInterceptPoint(P1, P2: TCoord): boolean;
    function  PointInTriangle(AnglePoint1, AnglePoint2, AnglePoint3: TCoord): boolean;
    function  TransformPolygon: boolean;
  public
    CouldNotResolve: boolean;
    Points: array[0..50] of TPoint;
    AngleCount: integer;
    VehiclePosition: TPoint;
    function IsDrivingOutsidePolygon: boolean;
  end;

implementation

procedure TPolygon.ReadCoordinates;
var
   i: integer;
begin
   for i := 0 to AngleCount - 1 do begin
      Polygon[i].X := Angles.Points[i].X;
      Polygon[i].Y := Angles.Points[i].Y;
   end;
end;

procedure TPolygon.RemoveAngle(AngleNo: integer);
var
   Angle: integer;
begin
   for Angle := AngleNo + 1 to AngleCount - 1 do
      Polygon[Angle - 1] := Polygon[Angle];
   Dec(AngleCount);
end;

function TPolygon.UpperXAxis(P1, P2: TCoord): boolean;
begin
   if P1.X < P2.X then
      Result := (P1.Y - P2.Y) * P1.X >= P1.Y * (P1.X - P2.X)
   else
      Result := (P1.Y - P2.Y) * P1.X <= P1.Y * (P1.X - P2.X);
end;

function TPolygon.YInterceptPoint(P1, P2: TCoord): boolean;
begin
   if P1.X = P2.X then
      Result := False
   else
      if P1.X < 0 then
         Result := P2.X >= 0
      else
         Result := P2.X <= 0;
end;

function TPolygon.PointInTriangle(AnglePoint1, AnglePoint2, AnglePoint3: TCoord): boolean;
var
   P1, P2, P3: TCoord;
   Intercept1, Intercept2, Intercept3, A, B, ItIsIn: boolean;
begin
   P1.X := AnglePoint1.X - SearchPoint.X;
   P2.X := AnglePoint2.X - SearchPoint.X;
   P3.X := AnglePoint3.X - SearchPoint.X;
   P1.Y := AnglePoint1.Y - SearchPoint.Y;
   P2.Y := AnglePoint2.Y - SearchPoint.Y;
   P3.Y := AnglePoint3.Y - SearchPoint.Y;
   Intercept1 := YInterceptPoint(P1, P2);
   Intercept2 := YInterceptPoint(P2, P3);
   Intercept3 := YInterceptPoint(P3, P1);

   ItIsIn := False;
   if Intercept1 and Intercept2 then begin
      A := UpperXAxis(P1, P2);
      B := UpperXAxis(P2, P3);
      if (A xor B) then
         ItIsIn := True;
   end;

   if Intercept1 and Intercept3 and not ItIsIn then begin
      A := UpperXAxis(P1, P2);
      B := UpperXAxis(P3, P1);
      if (A xor B) then
         ItIsIn := True;
   end;

   if Intercept2 and Intercept3 and not ItIsIn then begin
      A := UpperXAxis(P2, P3);
      B := UpperXAxis(P3, P1);
      if (A xor B) then
         ItIsIn := True;
   end;

   Result := ItIsIn;
end;

function TPolygon.TransformPolygon: boolean;
var
   AngleGone: boolean;
	Angle1, Angle2, Angle3: integer;
begin
	if AngleCount > 3 then begin
      Angle1 := 0;
      while (Angle1 < AngleCount) and (AngleCount > 3) do begin
         AngleGone := true;
         while AngleGone and (AngleCount > 3) do begin
            if Angle1 = AngleCount - 1 then begin
               Angle2 := 0;
               Angle3 := 1;
            end
            else if Angle1 + 1 = AngleCount - 1 then begin
               Angle2 := AngleCount - 1;
               Angle3 := 0;
            end
            else begin
               Angle2 := Angle1 + 1;
               Angle3 := Angle1 + 2;
            end;
            AngleGone := not PointInTriangle(Polygon[Angle1], Polygon[Angle2], Polygon[Angle3]);
            if AngleGone then
               RemoveAngle(Angle2);
         end;
         Angle1 := Angle1 + 1;
      end;
   end;
   Result := AngleCount = 3;
end;

function TPolygon.IsDrivingOutsidePolygon: boolean;
var
   n: integer;
begin
   CouldNotResolve := False;
   SearchPoint.Y := VehiclePosition.Y;
   SearchPoint.X := VehiclePosition.X;
   for n := 0 to AngleCount - 1 do begin
      Angles.Points[n].X := Points[n].X;
      Angles.Points[n].Y := Points[n].Y;
   end;
   ReadCoordinates;
   Angles.NoOfPoints := AngleCount;
   if (TransformPolygon) then
      Result := not PointInTriangle(Polygon[0], Polygon[1], Polygon[2])
   else begin
      Result := False; // transformation not possible
      CouldNotResolve := True;
   end;
end;

end.
